<?php

namespace Drupal\block_inactive_users;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Component\Datetime\TimeInterface;
use Drupal\user\Entity\User;
use Drupal\user\UserInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\Core\Mail\Plugin\Mail\PhpMail;
use Drupal\Core\Url;
use Drupal\Core\Language\LanguageManagerInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;

/**
 * Class InactiveUsersHandler.
 *
 * @package Drupal\block_inactive_users
 */
class InactiveUsersHandler {
  use StringTranslationTrait;

  const FORM_SETTINGS_CONFIG_OBJ_NAME = "block_inactive_users.settings";
  const LOGGER_CHANNEL = "block_inactive_users";

  /**
   * A config factory service.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactoryService;

  /**
   * A logger instance.
   *
   * @var \Drupal\Core\Logger\LoggerChannelFactoryInterface
   */
  protected $logger;

  /**
   * An language manager instance.
   *
   * @var \Drupal\Core\Language\LanguageManagerInterface
   */
  protected $languageManager;

  /**
   * A time service.
   *
   * @var \Drupal\Component\Datetime\TimeInterface
   */
  protected $timeService;

  /**
   * Constructor.
   */
  public function __construct(ConfigFactoryInterface $config_factory,
                              LoggerChannelFactoryInterface $loggerChannelFactory,
                              LanguageManagerInterface $language_manager,
                              TimeInterface $time_service) {

    $this->configFactoryService = $config_factory;
    $this->logger = $loggerChannelFactory;
    $this->languageManager = $language_manager;
    $this->timeService = $time_service;

  }

  /**
   * Disables an inactive user account.
   *
   * @param \Drupal\user\Entity\User $user
   *   A user object.
   * @param bool $sendmail
   *   Boolean to trigger email.
   *
   * @return \Drupal\user\Entity\User
   *   Returns a blocked user object.
   *
   * @throws \Drupal\Core\Entity\EntityStorageException
   */
  public function disableInactiveUsersStatus(User $user, $sendmail = TRUE) {

    $user->block();
    $user->save();
    $this->logger->get(self::LOGGER_CHANNEL)->info($user->getAccountName() . ' has been disabled.');

    if ($sendmail) {

      $activation_link = $this->createReactivationUrl($user);

      // Sends email notification.
      $this->mailUser($this->configFactoryService->get(self::FORM_SETTINGS_CONFIG_OBJ_NAME)
        ->get('block_inactive_users_from_email'),
        $user->getEmail(), $user->getAccountName(),
        $this->configFactoryService->get(self::FORM_SETTINGS_CONFIG_OBJ_NAME)
          ->get('block_inactive_users_email_subject'),
        $this->configFactoryService->get(self::FORM_SETTINGS_CONFIG_OBJ_NAME)
          ->get('block_inactive_users_email_content'),
        $activation_link);
    }

    return $user;

  }

  /**
   * Takes 2 timestamps and returns the month interval as a string.
   *
   * @param string $last_access
   *   A timestamp of the last time a user accessed their account.
   * @param string $current_time
   *   time(), Value of the current timestamp.
   *
   * @return int
   *   Returns the interval of the times in months.
   *
   * @throws \Exception
   */
  public function timestampdiff($last_access, $current_time) {

    $begin = new \DateTime("@$last_access");
    $end = new \DateTime("@$current_time");

    $interval = $begin->diff($end);

    return ($interval->y * 12) + ($interval->m);
  }

  /**
   * Sends a email notification.
   *
   * @param string $from
   *   From email address.
   * @param string $to
   *   To email address.
   * @param string $user_name
   *   Users first name.
   * @param string $message_subject
   *   Message subject field.
   * @param string $message_content
   *   Email template.
   * @param string $activation_link
   *   Url to re-activate user.
   *
   * @return bool
   *   Returns true if mail was sent successfully.
   */
  public function mailUser($from, $to, $user_name, $message_subject, $message_content, $activation_link) {

    $message = [];

    $system_site_config = $this->configFactoryService->get('system.site');
    $site_name = $system_site_config->get('name');
    $send_mail = new PhpMail();

    $message['headers'] = [
      'content-type' => 'text/html',
      'MIME-Version' => '1.0',
      'reply-to' => $from,
      'from' => $site_name . ' <' . $from . '>',
    ];

    $message['to'] = $to;

    $tring_to_replace = [
      '[account-name]' => $user_name,
      '[site-name]' => $site_name,
      '[activation-link]' => $activation_link,
    ];

    $message['subject'] = $message_subject;

    foreach ($tring_to_replace as $key => $value) {
      $message_content = str_replace($key, $value, $message_content);
    }
    $message['body'] = $message_content;

    $result = $send_mail->mail($message);
    if ($result != TRUE) {
      $message = $this->t('There was a problem sending your email notification to @email.', ['@email' => $to]);

      $this->logger->get('mail-log')->error($message);
      return FALSE;
    }
    $message = $this->t('An email notification has been sent to @email.', ['@email' => $to]);
    $this->logger->get('mail-log')->notice($message);

    return FALSE;

  }

  /**
   * Generates a URL to confirm an account reactivation request.
   *
   * @param \Drupal\user\UserInterface $account
   *   The user account object.
   * @param array $options
   *   (optional) A keyed array of settings. Supported options are:
   *   - langcode: A language code to be used when generating locale-sensitive
   *     URLs. If langcode is NULL the users preferred language is used.
   *
   * @return string
   *   A unique URL that may be used to confirm the reactivation of the user
   *   account.
   */
  private function createReactivationUrl(UserInterface $account, array $options = []) {

    $timestamp = $this->timeService->getRequestTime();
    $langcode = isset($options['langcode']) ? $options['langcode'] : $account->getPreferredLangcode();
    $url_options = ['absolute' => TRUE, 'language' => $this->languageManager->getLanguage($langcode)];
    return Url::fromRoute('block_inactive_users.reactivate_confirm', [
      'user' => $account->id(),
      'timestamp' => $timestamp,
      'hashed_pass' => user_pass_rehash($account, $timestamp),
    ], $url_options)->toString();

  }

  /**
   * Takes 3 paramates to cancel user depending on rules setup.
   *
   * @param string $uid
   *   Uid of the user account being canceled.
   * @param string $block_inactive_users_cancel_email
   *   Block inactive users cancel email.
   * * @param string $disable_account_method
   *   Disable account method.
   *
   * @throws \Exception
   */
  public function cancelUser( $uid, $block_inactive_users_cancel_email, $disable_account_method)
  {
    user_cancel(array(
      'user_cancel_notify' => $block_inactive_users_cancel_email,
      'user_cancel_method' => $disable_account_method,
    ), $uid, $disable_account_method);
  }
}
